@@ -1,3 +1,4 @@
+
 //
 // Beagle
 //
@@ -37,6 +38,7 @@ using System.Collections;
 using System.IO;
 using System.Text;
 using Beagle.Util;
+
 internal class RTFControlWordType {
 	
 	public enum Type {
@@ -45,6 +47,8 @@ internal class RTFControlWordType {
 		MetaDataBlock,
 		MetaDataTag,
 		Paragraph,
+		ParaEnd,
+		SplSection,
 		EscSeq,
 		CharProp
 	}
@@ -72,8 +76,13 @@ internal class RTFControlWordType {
 		new RTFControlWordType (Type.MetaDataTag, "nofwords"),
 		new RTFControlWordType (Type.MetaDataTag, "generator"),
 		new RTFControlWordType (Type.MetaDataTag, "company"),
-		new RTFControlWordType (Type.Paragraph, "par"),
+		new RTFControlWordType (Type.ParaEnd, "par"),
 		new RTFControlWordType (Type.Paragraph, "pard"),
+		new RTFControlWordType (Type.SplSection, "header"),
+		new RTFControlWordType (Type.SplSection, "footer"),
+		new RTFControlWordType (Type.SplSection, "headerl"),
+		new RTFControlWordType (Type.SplSection, "footerl"),
+		new RTFControlWordType (Type.SplSection, "footnote"),
 		new RTFControlWordType (Type.CharProp, "b"),
 		new RTFControlWordType (Type.CharProp, "i"),
 		new RTFControlWordType (Type.CharProp, "ul"),
@@ -103,7 +112,8 @@ namespace Beagle.Filters {
 			None,
 			InMetaData,
 			InMetaDataTagGenerator,
-			InBody
+			InBody,
+			InPara
 		}
 
 		public enum ErrorCodes {
@@ -113,31 +123,52 @@ namespace Beagle.Filters {
 		};
 		
 		Position pos;
-		Stack MetaDataStack;
-		Stack TextDataStack;
 		int groupCount;
-		long offset;
+		int skipCount;
+		int hotStyleCount;
+		bool bPartHotStyle;
 		FileStream FsRTF;
 		StreamReader SReaderRTF;
+	        string partText;
+
+		Stack MetaDataStack;
+		Stack TextDataStack;
 
 		public FilterRTF ()
 		{
 			// Make this a general rtf filter.
 			AddSupportedMimeType ("application/rtf");
+
 			pos = Position.None;
-			MetaDataStack = new Stack();
-			TextDataStack = new Stack();
 			groupCount = 0;
-			offset = 0;
+			skipCount = 0;
+			hotStyleCount = 0;
+			bPartHotStyle = false;
 			FsRTF = null;
 			SReaderRTF = null;
+			partText = "";
+
+			MetaDataStack = new Stack ();
+			TextDataStack = new Stack ();
+
+			SnippetMode = true;
 		}
 
 		override protected void DoOpen (FileInfo info) 
 		{
-			FsRTF = new FileStream (info.FullName, FileMode.Open, FileAccess.Read);
-			if (FsRTF != null)
-				SReaderRTF = new StreamReader (FsRTF);
+			try {
+				FsRTF = new FileStream (info.FullName, FileMode.Open, 
+							FileAccess.Read);
+				if (FsRTF != null)
+					SReaderRTF = new StreamReader (FsRTF);
+				else {
+					Logger.Log.Error ("Unable to open {0}.", info.FullName);
+					Finished ();
+				}
+			} catch (Exception e) {
+				Logger.Log.Error ("Unable to open {0}.", info.FullName);
+				Finished ();
+			}
 			
 		}
 
@@ -177,25 +208,43 @@ namespace Beagle.Filters {
 				break;
 
 			case RTFControlWordType.Type.Paragraph:
+				if (!bMeta)
+					pos = Position.InPara;
+				break;
+
+			case RTFControlWordType.Type.ParaEnd:
 				if (!bMeta)
 					pos = Position.InBody;
 				break;
 
 				// FIXME: "Hot" styles are not *properly reset to normal*
 				// on some *wierd* conditions.
+				// To avoid such stuff, we need to maintain a stack of 
+				// groupCounts for set/reset Hot styles.
+			case RTFControlWordType.Type.SplSection:
+				hotStyleCount = groupCount - 1;
+				break;
+
 			case RTFControlWordType.Type.CharProp:
-				if (pos == Position.InBody) {
-					if (paramVal < 0)
-						HotUp ();
+				if (pos == Position.InPara) {
+					if (paramVal < 0) {
+						//Console.WriteLine ("HotUp: \\{0}{1}", strCtrlWord, paramVal);
+						hotStyleCount = groupCount - 1;
+						//HotUp ();
+					}
 				}
 				break;
 
 			case RTFControlWordType.Type.EscSeq:
-				if (pos == Position.InBody) {
+				if (pos == Position.InPara) {
 					TextDataStack.Push (strCtrlWord);
 					TextDataStack.Push ("EscSeq");
 				}
 				break;
+			case RTFControlWordType.Type.Skip:
+				skipCount = groupCount - 1;
+				//SkipDataStack.Push (groupCount-1);
+				break;
 			}
 			return ErrorCodes.ERROR_RTF_OK;
 		}
@@ -206,7 +255,8 @@ namespace Beagle.Filters {
 		{
 			int aByte = -1;
 			char ch;
-			int paramVal = -1, i;
+			int paramVal = -1;
+			bool negParamVal = false;
 			StringBuilder strCtrlWord = new StringBuilder ();
 			StringBuilder strParameter = new StringBuilder ();
 			
@@ -220,7 +270,7 @@ namespace Beagle.Filters {
 			if (!Char.IsLetter (ch) && 
 			    ctrlWrdType.Types != RTFControlWordType.Type.Skip &&
 			    ctrlWrdType.Types != RTFControlWordType.Type.EscSeq) {
-				Console.WriteLine ("Unhandled symbol: {0}, {1}", ch, ctrlWrdType.Types);
+				Logger.Log.Error ("Unhandled symbol: {0}, {1}", ch, ctrlWrdType.Types);
 				return ErrorCodes.ERROR_RTF_UNHANDLED_SYMBOL;
 			}
 			while (aByte != -1) {
@@ -236,6 +286,12 @@ namespace Beagle.Filters {
 			}
 			aByte = SReaderRTF.Peek ();
 			ch = (char) aByte;
+			if (aByte != -1 && ch == '-') {
+				negParamVal = true;
+				aByte = SReaderRTF.Read (); // move the fp
+				aByte = SReaderRTF.Peek ();
+				ch = (char) aByte;
+			}
 			if (Char.IsDigit (ch)) {
 				aByte = SReaderRTF.Read ();
 				ch = (char) aByte;
@@ -254,6 +310,8 @@ namespace Beagle.Filters {
 					paramVal = Convert.ToInt32 (strParameter.ToString());
 			}
 			//Console.WriteLine ("{0}\t{1}", strCtrlWord, strParameter);
+			if (negParamVal && paramVal > -1)
+				paramVal *= -1;
 			return (HandleControlWord (strCtrlWord.ToString(), paramVal, bMeta));
 		}
 
@@ -264,29 +322,27 @@ namespace Beagle.Filters {
 			StringBuilder str = new StringBuilder ();
 			string strTemp = null;
 			ErrorCodes ec;
-
-			// If we are not extracting meta-data, set the 
-			// file pointer to the saved position
-			if (!bMeta)
-				SReaderRTF.BaseStream.Seek (offset, SeekOrigin.Begin);
 		       
 			while ((aByte = SReaderRTF.Read ()) != -1) {
 				ch = (char) aByte;
 				switch (ch) {
 				case '\\': /* process keywords */
+					if (skipCount > 0) {
+						if (groupCount > skipCount)
+							continue;
+						else
+							skipCount = 0;
+					}
 					ec = ProcessControlWords (bMeta); 
 					if (ec != ErrorCodes.ERROR_RTF_OK)
 						return ec;
-					if (pos == Position.InBody) {
+					if (pos == Position.InPara)
 						AddTextForIndexing (str);
-						//AppendText (str.ToString());
-						//AppendWhiteSpace ();
-					}
 					str.Remove (0, str.Length);
 					break;
 				case '{': /* process groups */
-					if (pos == Position.InBody)
-					    AddTextForIndexing (str);
+					if (pos == Position.InPara)
+						AddTextForIndexing (str);
 					str.Remove (0, str.Length);
 					groupCount++;
 					break;
@@ -297,11 +353,8 @@ namespace Beagle.Filters {
 						// groupCount will atleast be 1 for 
 						// the outermost "{" block
 						if (pos == Position.InMetaData && groupCount == 1) {
-							if (bMeta) {
-								offset = SReaderRTF.BaseStream.Position;
+							if (bMeta)
 								return ErrorCodes.ERROR_RTF_OK;
-							}
-
 						} else {
 							if (MetaDataStack.Count > 0) {
 								strTemp = (string) MetaDataStack.Pop ();
@@ -317,31 +370,51 @@ namespace Beagle.Filters {
 							}
 						}
 						
+					} else if (pos == Position.InPara) {
+						AddTextForIndexing (str);
+
 					} else if (pos == Position.InBody) {
+						//Console.WriteLine ("\\par : {0}", str);
 						if (str.Length > 0)
 							str.Append (' ');
 						AddTextForIndexing (str);
-						if (IsHot)
-							HotDown ();
+						AppendStructuralBreak ();
 					}
-
+					if (hotStyleCount > 0
+					    && groupCount <= hotStyleCount) {
+						//Console.WriteLine ("Group count: {0}, stack: {1}", 
+						//groupCount, hotStyleCount);
+						HotDown ();
+						hotStyleCount = 0;
+					}
+					
 					break;
 				case '\r': /* ignore \r */
 				case '\n': /* ignore \n */
 					break;
 				default:
-					str.Append (ch);
+					if ((skipCount == 0 || groupCount <= skipCount)
+					    && (pos == Position.InPara || pos == Position.InBody))
+						str.Append (ch);
 					break;
 				}
 			}
+			if (partText.Length > 0) {
+				if (bPartHotStyle && !IsHot) 
+					HotUp ();
+				AppendText (partText);
+				if (IsHot)
+					HotDown ();
+			}
 			return ErrorCodes.ERROR_RTF_OK;
 		}
 
 		private void AddTextForIndexing (StringBuilder str)
 		{
 			string strTemp;
-			string strStyle;
-			int elemCount;
+			string paramStr = null;
+
+			bool wasHot = false;
 
 			while (TextDataStack.Count > 0) {
 				strTemp = (string) TextDataStack.Pop ();
@@ -352,17 +425,87 @@ namespace Beagle.Filters {
 					break;
 				}
 			}
+			
+			strTemp = "";
 			if (str.Length > 0) {
-				AppendText (str.ToString());
+				//Console.WriteLine ("Text: [{0}]", str);
+
+				paramStr = str.ToString ();
 				str.Remove (0, str.Length);
+
+				int index = paramStr.LastIndexOf (' ');
+				int sindex = 0;
+
+				if (index > -1) {
+					// During the previous-parsing, a word got terminatted partially,
+					// find the remaining part of the word, concatenate it and add it to 
+					// the respective pools and reset the HOT status, if required.
+					if (partText.Length > 0) {
+						sindex = paramStr.IndexOf (' ');
+						strTemp = partText + paramStr.Substring (0, sindex);
+						//Console.WriteLine ("PartHotStyle: {0}, HotStyleCount: {1}, partText: {2}",
+						//   bPartHotStyle,
+						//	   hotStyleCount, strTemp);
+						if (!IsHot) {
+							if (bPartHotStyle)
+								HotUp ();
+						}
+						else
+							wasHot = true;
+
+						AppendText (strTemp);
+						if (!wasHot && bPartHotStyle)
+							HotDown ();
+						bPartHotStyle = false;
+					}
+					paramStr = paramStr.Substring (sindex);
+					index = paramStr.LastIndexOf (' ');
+					sindex = 0;
+				}
+				if (index > -1) {
+					partText = paramStr.Substring (index);
+					paramStr = paramStr.Substring (sindex, index);
+				} else {
+					strTemp = partText + paramStr;
+					partText = strTemp;
+					paramStr = "";
+					strTemp = "";
+				}
+					
+				// Enable *HOT* just before appending the text
+				// because, there can be some *Partial Texts* without
+				// *HOT* styles that needs to be appended.
+				if (hotStyleCount > 0) {
+					if (!IsHot)
+						HotUp ();
+					bPartHotStyle = true;
+				} else 
+					bPartHotStyle |= false;
+
+				if (paramStr.Length > 0)
+					AppendText (paramStr);
+
+				if (partText.Length < 1)
+					bPartHotStyle = false;
 			}
 		}
-			
+
 		override protected void DoPull ()
 		{
 			ErrorCodes ec;
 			ec = ErrorCodes.ERROR_RTF_OK;
 			pos = Position.None;
+
+			// Discard the buffered data, if not,
+			// the buffered data can change the 
+			// state "pos" variable that results 
+			// in complete mess.
+			// Fixes: http://bugzilla.gnome.org/show_bug.cgi?id=172294
+			SReaderRTF.DiscardBufferedData ();
+
+			// Rewind the file pointer to start from beginning.
+			SReaderRTF.BaseStream.Seek (0, SeekOrigin.Begin);
+
 			ec = RTFParse (false);
 			if (ec != ErrorCodes.ERROR_RTF_OK)
 				Logger.Log.Error ("{0}", ec);
