@@ -45,6 +45,9 @@ namespace Beagle.Daemon {
 
 		private static Server server = null;
 
+		private static bool arg_replace = false;
+		private static bool arg_disable_scheduler = false;
+
 		public static bool StartServer ()
 		{
 			Logger.Log.Debug ("Starting messaging server");
@@ -114,26 +117,81 @@ namespace Beagle.Daemon {
 
 #if ENABLE_WEBSERVICES
 			usage += "\n" +
-				"  --web-global\t\tAllow global access to the web interface.\n" +
-				"  --web-start\t\tStart the web server interface for this instance..n" +
-				"  --web-port\t\tPort to use for the web server interface.\n" +
-				"  --web-root\t\tRoot of the web server interface.\n";
+				"  --web-global\t\tAllow global access to the Web & WebService interfaces.\n" +
+				"  --web-start\t\tStart internal web server for this instance.\n" +
+				"  --web-port\t\tPort to use for the internal web server.\n" +
+				"  --web-root\t\tRoot directory to use for the internal web server.\n" +
+				"  --enable-network\tStart internal web server & allow global access.\n";
 #endif 
 
 			Console.WriteLine (usage);
 		}
 
+		public static bool StartupProcess ()
+		{
+			// Profile our initialization
+			Stopwatch stopwatch = new Stopwatch ();
+			stopwatch.Start ();
+
+			SetupSignalHandlers ();
+
+			// Fire up our server
+			if (! StartServer ()) {
+				if (arg_replace)
+					ReplaceExisting ();
+				else {
+					Logger.Log.Fatal ("Could not set up the listener for beagle requests.  "
+							  + "There is probably another beagled instance running.  "
+							  + "Use --replace to replace the running service");
+					Environment.Exit (1);
+				}
+			}
+			
+			// Set up out-of-process indexing
+			if (Environment.GetEnvironmentVariable ("BEAGLE_ENABLE_IN_PROCESS_INDEXING") == null)
+				LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
+
+			// Start the query driver.
+			Logger.Log.Debug ("Starting QueryDriver");
+			QueryDriver.Start ();
+
+			// Start the Global Scheduler thread
+			if (! arg_disable_scheduler) {
+				Logger.Log.Debug ("Starting Scheduler thread");
+				Scheduler.Global.Start ();
+			}
+
+			// Start our Inotify threads
+			Inotify.Start ();
+	
+			// Test if the FileAdvise stuff is working: This will print a
+			// warning if not.  The actual advice calls will fail silently.
+			FileAdvise.TestAdvise ();
+
+#if ENABLE_WEBSERVICES		
+			//Beagle Web, WebService access initialization code:
+			WebServiceBackEnd.Start();
+#endif
+			Shutdown.ShutdownEvent += OnShutdown;
+
+			// Load user configuration
+			Conf.Load ();
+			Conf.WatchForUpdates ();
+
+			stopwatch.Stop ();
+
+			Logger.Log.Debug ("Daemon initialization finished after {0}", stopwatch);
+		
+			return false;
+		}
+
 		public static int Main (string[] args)
 		{
 			// Process the command-line arguments
-			bool arg_replace = false;
 			bool arg_debug = false;
 			bool arg_debug_memory = false;
 			bool arg_fg = false;
-			bool arg_disable_scheduler = false;
-#if ENABLE_WEBSERVICES
-			WebServicesArgs wsargs = new WebServicesArgs();
-#endif
+
 			int i = 0;
 			while (i < args.Length) {
 				
@@ -194,24 +252,29 @@ namespace Beagle.Daemon {
 					break;
 #if ENABLE_WEBSERVICES
 				case "--web-global":
-					wsargs.web_global = true;
+					WebServiceBackEnd.web_global = true;
 					break;
 
 				case "--web-start":
-					wsargs.web_start = true;
+					WebServiceBackEnd.web_start = true;
 					break;
 
 				case "--web-port":
-					wsargs.web_port = next_arg;
+					WebServiceBackEnd.web_port = next_arg;
 					++i; 
-					wsargs.web_start = true;
+					WebServiceBackEnd.web_start = true;
 					break;
 
 				case "--web-root":
-					wsargs.web_rootDir = next_arg;
+					WebServiceBackEnd.web_rootDir = next_arg;
 					++i; 
-					wsargs.web_start = true;
+					WebServiceBackEnd.web_start = true;
 					break;
+					
+				case "--enable-network":
+					WebServiceBackEnd.web_start = true;
+					WebServiceBackEnd.web_global = true;
+					break;					
 #endif 
 				default:
 					Console.WriteLine ("Ignoring unknown argument '{0}'", arg);
@@ -253,69 +316,25 @@ namespace Beagle.Daemon {
 			if (! ExtendedAttribute.Test (PathFinder.StorageDir)) {
 				Logger.Log.Fatal ("Could not set extended attributes on a file in your home "
 						  + "directory.  See "
-						  + "http://www.beaglewiki.org/index.php/Enable%20Extended%20Attributes "
+						  + "http://www.beaglewiki.org/Enabling_Extended_Attributes "
 						  + "for more information.");
 				return 1;
 			}
 
 			// Start our memory-logging thread
-			if (arg_debug_memory) {
-				Thread th = new Thread (new ThreadStart (LogMemoryUsage));
-				th.Start ();
-			}
+			if (arg_debug_memory)
+				ExceptionHandlingThread.Start (new ThreadStart (LogMemoryUsage));
+
+			// Do BEAGLE_EXERCISE_THE_DOG_HARDER-related processing.
+			ExerciseTheDogHarder ();
 
 			Application.InitCheck ("beagled", ref args);
 
-			// Profile our initialization
-			Stopwatch stopwatch = new Stopwatch ();
-			stopwatch.Start ();
+			// Defer all actual startup until the main loop is
+			// running.  That way shutdowns during the startup
+			// process work correctly.
+			GLib.Idle.Add (new GLib.IdleHandler (StartupProcess));
 
-			SetupSignalHandlers ();
-
-			// Fire up our server
-			if (! StartServer ()) {
-				if (arg_replace)
-					ReplaceExisting ();
-				else {
-					Logger.Log.Fatal ("Could not set up the listener for beagle requests.  "
-							  + "There is probably another beagled instance running.  "
-							  + "Use --replace to replace the running service");
-					return 1;
-				}
-			}
-			
-			// Set up out-of-process indexing
-			if (Environment.GetEnvironmentVariable ("BEAGLE_ENABLE_IN_PROCESS_INDEXING") == null)
-				LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
-
-			// Start the query driver.
-			Logger.Log.Debug ("Starting QueryDriver");
-			QueryDriver.Start ();
-
-			// Start the Global Scheduler thread
-			if (! arg_disable_scheduler) {
-				Logger.Log.Debug ("Starting Scheduler thread");
-				Scheduler.Global.Start ();
-			}
-
-			// Start our Inotify threads
-			Logger.Log.Debug ("Starting Inotify threads");
-			Inotify.Start ();
-	
-			// Test if the FileAdvise stuff is working: This will print a
-			// warning if not.  The actual advice calls will fail silently.
-			FileAdvise.TestAdvise ();
-
-#if ENABLE_WEBSERVICES		
-			//Beagle Web, WebService access initialization code:
-			WebServiceBackEnd.Start(wsargs);
-#endif
-			Shutdown.ShutdownEvent += OnShutdown;
-
-			stopwatch.Stop ();
-
-			Logger.Log.Debug ("Daemon initialization finished after {0}", stopwatch);
-		
 			// Start our event loop.
 			Logger.Log.Debug ("Starting main loop");
 
@@ -323,6 +342,11 @@ namespace Beagle.Daemon {
 
 			Logger.Log.Debug ("Leaving BeagleDaemon.Main");
 
+			if (Logger.Log.Level == LogLevel.Debug) {
+				Thread.Sleep (500);
+				ExceptionHandlingThread.SpewLiveThreads ();
+			}
+
 			return 0;
 		}
 		
@@ -409,5 +433,62 @@ namespace Beagle.Daemon {
 			// Stop the messaging server
 			server.Stop ();
 		}
+
+		/////////////////////////////////////////////////////////////////////////////
+
+
+		private static ArrayList exercise_files = new ArrayList ();
+
+		private static void ExerciseTheDogHarder ()
+		{
+			string path;				
+			path = Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG_HARDER");
+			if (path == null)
+				return;
+
+			DirectoryInfo dir = new DirectoryInfo (path);
+			foreach (FileInfo file in dir.GetFiles ())
+				exercise_files.Add (file);
+			if (exercise_files.Count == 0)
+				return;
+
+			int N = 5;
+			if (N > exercise_files.Count)
+				N = exercise_files.Count;
+			
+			for (int i = 0; i < N; ++i)
+				ExceptionHandlingThread.Start (new ThreadStart (ExerciseTheDogHarderWorker));
+		}
+
+		private static void ExerciseTheDogHarderWorker ()
+		{
+			Random rng = new Random ();
+
+			while (! Shutdown.ShutdownRequested) {
+
+				FileInfo file = null;			
+				int i;
+
+			
+				lock (exercise_files) {
+					do {
+						i = rng.Next (exercise_files.Count);
+						file = exercise_files [i] as FileInfo;
+					} while (file == null);
+					exercise_files [i] = null;
+				}
+
+				string target;
+				target = Path.Combine (PathFinder.HomeDir, "_HARDER_" + file.Name);
+
+				Logger.Log.Debug ("ETDH: Copying {0}", file.Name);
+				file.CopyTo (target, true);
+				
+				lock (exercise_files)
+					exercise_files [i] = file;
+
+				Thread.Sleep (500 + rng.Next (500));
+			}
+		}
 	}
 }
