@@ -63,32 +63,50 @@ namespace Lucene.Net.Store
 					tmpBool = System.IO.Directory.Exists(Enclosing_Instance.lockDir.FullName);
 				if (!tmpBool)
 				{
-                    try
-                    {
-                        System.IO.Directory.CreateDirectory(Enclosing_Instance.lockDir.FullName);
-                    }
-                    catch (Exception)
+					try
+					{
+						System.IO.Directory.CreateDirectory(Enclosing_Instance.lockDir.FullName);
+					}
+					catch (Exception)
 					{
 						throw new System.IO.IOException("Cannot create lock directory: " + Enclosing_Instance.lockDir);
 					}
 				}
 				
-                try
-                {
-                    System.IO.FileStream createdFile = lockFile.Create();
-                    createdFile.Close();
-		    // ADDED joeshaw@novell.com 10 Jan 2005  Lock debugging
-		    Log ("Obtained lock {0}", lockFile.FullName);
-                    return true;
-                }
-                catch (Exception e)
-                {
-			// ADDED joeshaw@novell.com 10 Jan 2005  Lock debugging
-			Log ("Could not obtain lock {0}", lockFile.FullName);
-			Log (e);
-			return false;
-                }
+				// ADDED trow@novell.com 17 Mar 2005: Use Mono.Posix for lock creation
+
+				bool obtainedLock = false;
+
+				int fd = Mono.Posix.Syscall.open (lockFile.FullName,
+								  Mono.Posix.OpenFlags.O_CREAT
+								  | Mono.Posix.OpenFlags.O_EXCL,
+								  Mono.Posix.FileMode.S_IRUSR
+								  | Mono.Posix.FileMode.S_IWUSR);
+
+				if (fd != -1) {
+					Mono.Posix.Syscall.close (fd);
+					lockFile.Refresh ();
+					obtainedLock = true;
+				}
+			
+				
+#if false
+				try
+				{
+					System.IO.FileStream createdFile = lockFile.Create();
+					createdFile.Close();
+					obtainedLock = true;
+				}
+				catch (Exception e)
+				{
+					// Just fall through
+				}
+#endif
+
+				Log ("{0} lock {1}", obtainedLock ? "Obtained" : "Could not obtain", lockFile.FullName);
+				return obtainedLock;
 			}
+
 			public override void  Release()
 			{
 				if (Lucene.Net.Store.FSDirectory.DISABLE_LOCKS)
@@ -184,9 +202,16 @@ namespace Lucene.Net.Store
 		/// </param>
 		/// <returns> the FSDirectory for the named file.  
 		/// </returns>
+
+		/// CHANGED trow@novell.com 17 Mar 2005 set tmp dir at creation-time
+		public static FSDirectory GetDirectory(System.String path, System.String tmpDir, bool create)
+		{
+			return GetDirectory(new System.IO.FileInfo(path), tmpDir, create);
+		}
+
 		public static FSDirectory GetDirectory(System.String path, bool create)
 		{
-			return GetDirectory(new System.IO.FileInfo(path), create);
+			return GetDirectory(new System.IO.FileInfo(path), null, create);
 		}
 		
 		/// <summary>Returns the directory instance for the named location.
@@ -202,7 +227,7 @@ namespace Lucene.Net.Store
 		/// </param>
 		/// <returns> the FSDirectory for the named file.  
 		/// </returns>
-		public static FSDirectory GetDirectory(System.IO.FileInfo file, bool create)
+		public static FSDirectory GetDirectory(System.IO.FileInfo file, System.String tmpDir, bool create)
 		{
 			file = new System.IO.FileInfo(file.FullName);
 			FSDirectory dir;
@@ -211,7 +236,7 @@ namespace Lucene.Net.Store
 				dir = (FSDirectory) DIRECTORIES[file];
 				if (dir == null)
 				{
-					dir = new FSDirectory(file, create);
+					dir = new FSDirectory(file, tmpDir, create);
 					DIRECTORIES[file] = dir;
 				}
 				else if (create)
@@ -225,16 +250,24 @@ namespace Lucene.Net.Store
 			}
 			return dir;
 		}
+
+		public static FSDirectory GetDirectory(System.IO.FileInfo file, bool create)
+		{
+			return GetDirectory (file, null, create);
+		}
 		
 		private System.IO.FileInfo directory = null;
 		private int refCount;
 		private System.IO.FileInfo lockDir;
 		
-		private FSDirectory(System.IO.FileInfo path, bool create)
+		private FSDirectory(System.IO.FileInfo path, string tmpDir, bool create)
 		{
 			directory = path;
 			
 			// FIXED joeshaw@novell.com  10 Jan 2005  Use TempDirectoryName to find where locks live
+			// FIXED trow@novell.com  17 Mar 2005  A fix on the fix
+			if (tmpDir != null)
+				TempDirectoryName = tmpDir;
 			lockDir = new System.IO.FileInfo (TempDirectoryName);
 			if (create)
 			{
@@ -441,22 +474,26 @@ namespace Lucene.Net.Store
                         int len;
                         len = in_Renamed.Read(buffer, 0, buffer.Length);
                         out_Renamed.Write(buffer, 0, len);
+
+			// FIXED trow@novell.com 18 Feb 2005
+			// Commented out unused variables tmpBool3 and generatedAux
+			// to avoid compiler warnings
 						
                         // delete the old file.
-                        bool tmpBool3;
+                        // bool tmpBool3;
                         if (System.IO.File.Exists(old.FullName))
                         {
                             System.IO.File.Delete(old.FullName);
-                            tmpBool3 = true;
+                            // tmpBool3 = true;
                         }
                         else if (System.IO.Directory.Exists(old.FullName))
                         {
                             System.IO.Directory.Delete(old.FullName);
-                            tmpBool3 = true;
+                            // tmpBool3 = true;
                         }
-                        else
-                            tmpBool3 = false;
-                        bool generatedAux = tmpBool3;
+                        //else
+			//tmpBool3 = false;
+                        //bool generatedAux = tmpBool3;
                     }
                     catch (System.IO.IOException e)
                     {
@@ -507,6 +544,7 @@ namespace Lucene.Net.Store
 
 		// ADDED trow 4 June 2004
 		static public Beagle.Util.Logger Logger = null;
+		//static public Beagle.Util.Logger Logger = Beagle.Util.Logger.Log;
 		static private void Log (string format, params object[] args)
 		{
 			if (Logger != null)
@@ -776,4 +814,4 @@ namespace Lucene.Net.Store
 			file.Close(); // close the file
 		}
 	}
-}
\ No newline at end of file
+}
